import { ComputedGetter } from '@atom-vue/reactivity';
import { ComputedRef } from '@atom-vue/reactivity';
import { customRef } from '@atom-vue/reactivity';
import { DebuggerEvent } from '@atom-vue/reactivity';
import { isProxy } from '@atom-vue/reactivity';
import { isReactive } from '@atom-vue/reactivity';
import { isReadonly } from '@atom-vue/reactivity';
import { isRef } from '@atom-vue/reactivity';
import { markRaw } from '@atom-vue/reactivity';
import { reactive } from '@atom-vue/reactivity';
import { ReactiveEffect } from '@atom-vue/reactivity';
import { ReactiveEffectOptions } from '@atom-vue/reactivity';
import { readonly } from '@atom-vue/reactivity';
import { Ref } from '@atom-vue/reactivity';
import { ref } from '@atom-vue/reactivity';
import { shallowReactive } from '@atom-vue/reactivity';
import { shallowReadonly } from '@atom-vue/reactivity';
import { shallowRef } from '@atom-vue/reactivity';
import { toRaw } from '@atom-vue/reactivity';
import { toRef } from '@atom-vue/reactivity';
import { ToRefs } from '@atom-vue/reactivity';
import { toRefs } from '@atom-vue/reactivity';
import { TrackOpTypes } from '@atom-vue/reactivity';
import { TriggerOpTypes } from '@atom-vue/reactivity';
import { triggerRef } from '@atom-vue/reactivity';
import { unref } from '@atom-vue/reactivity';
import { UnwrapRef } from '@atom-vue/reactivity';
import { WritableComputedOptions } from '@atom-vue/reactivity';
import { WritableComputedRef } from '@atom-vue/reactivity';

export declare interface App<HostElement = any> {
    config: AppConfig;
    use(plugin: Plugin_2, ...options: any[]): this;
    mixin(mixin: ComponentOptions): this;
    component(name: string): PublicAPIComponent | undefined;
    component(name: string, component: PublicAPIComponent): this;
    directive(name: string): Directive | undefined;
    directive(name: string, directive: Directive): this;
    mount(rootContainer: HostElement | string, isHydrate?: boolean): ComponentPublicInstance;
    unmount(rootContainer: HostElement | string): void;
    provide<T>(key: InjectionKey<T> | string, value: T): this;
    _component: Component;
    _props: Data | null;
    _container: HostElement | null;
    _context: AppContext;
}

export declare interface AppConfig {
    readonly isNativeTag?: (tag: string) => boolean;
    devtools: boolean;
    performance: boolean;
    optionMergeStrategies: Record<string, OptionMergeFunction>;
    globalProperties: Record<string, any>;
    isCustomElement: (tag: string) => boolean;
    errorHandler?: (err: unknown, instance: ComponentPublicInstance | null, info: string) => void;
    warnHandler?: (msg: string, instance: ComponentPublicInstance | null, trace: string) => void;
}

export declare interface AppContext {
    config: AppConfig;
    mixins: ComponentOptions[];
    components: Record<string, PublicAPIComponent>;
    directives: Record<string, Directive>;
    provides: Record<string | symbol, any>;
    reload?: () => void;
}

export declare type AsyncComponentLoader<T = any> = () => Promise<AsyncComponentResolveResult<T>>;

export declare interface AsyncComponentOptions<T = any> {
    loader: AsyncComponentLoader<T>;
    loadingComponent?: PublicAPIComponent;
    errorComponent?: PublicAPIComponent;
    delay?: number;
    timeout?: number;
    suspensible?: boolean;
    onError?: (error: Error, retry: () => void, fail: () => void, attempts: number) => any;
}

declare type AsyncComponentResolveResult<T = PublicAPIComponent> = T | {
    default: T;
};

export declare const BaseTransition: new () => {
    $props: BaseTransitionProps<any>;
};

export declare interface BaseTransitionProps<HostElement = RendererElement> {
    mode?: 'in-out' | 'out-in' | 'default';
    appear?: boolean;
    persisted?: boolean;
    onBeforeEnter?: (el: HostElement) => void;
    onEnter?: (el: HostElement, done: () => void) => void;
    onAfterEnter?: (el: HostElement) => void;
    onEnterCancelled?: (el: HostElement) => void;
    onBeforeLeave?: (el: HostElement) => void;
    onLeave?: (el: HostElement, done: () => void) => void;
    onAfterLeave?: (el: HostElement) => void;
    onLeaveCancelled?: (el: HostElement) => void;
}

export declare function callWithAsyncErrorHandling(fn: Function | Function[], instance: ComponentInternalInstance | null, type: ErrorTypes, args?: unknown[]): any[];

export declare function callWithErrorHandling(fn: Function, instance: ComponentInternalInstance | null, type: ErrorTypes, args?: unknown[]): any;

/* Excluded from this release type: camelize */

declare interface ClassComponent {
    new (...args: any[]): ComponentPublicInstance<any, any, any, any, any>;
    __vccOpts: ComponentOptions;
}

export declare function cloneVNode<T, U>(vnode: VNode<T, U>, extraProps?: Data & VNodeProps): VNode<T, U>;

declare const Comment_2: unique symbol;
export { Comment_2 as Comment }

declare interface CompiledSlotDescriptor {
    name: string;
    fn: Slot;
}

export declare type Component = ComponentOptions | FunctionalComponent<any>;

/**
 * Interface for declaring custom options.
 *
 * @example
 * ```ts
 * declare module '@atom-vue/runtime-core' {
 *   interface ComponentCustomOptions {
 *     beforeRouteUpdate?(
 *       to: Route,
 *       from: Route,
 *       next: () => void
 *     ): void
 *   }
 * }
 * ```
 */
export declare interface ComponentCustomOptions {
}

/**
 * Custom properties added to component instances in any way and can be accessed through `this`
 *
 * @example
 * Here is an example of adding a property `$router` to every component instance:
 * ```ts
 * import { createApp } from 'vue'
 * import { Router, createRouter } from 'vue-router'
 *
 * declare module '@vue/runtime-core' {
 *   interface ComponentCustomProperties {
 *     $router: Router
 *   }
 * }
 *
 * // effectively adding the router to every component instance
 * const app = createApp({})
 * const router = createRouter()
 * app.config.globalProperties.$router = router
 *
 * const vm = app.mount('#app')
 * // we can access the router from the instance
 * vm.$router.push('/')
 * ```
 */
export declare interface ComponentCustomProperties {
}

declare type ComponentInjectOptions = string[] | Record<string | symbol, string | symbol | {
    from: string | symbol;
    default?: unknown;
}>;

/**
 * We expose a subset of properties on the internal instance as they are
 * useful for advanced external libraries and tools.
 */
export declare interface ComponentInternalInstance {
    uid: number;
    type: Component;
    parent: ComponentInternalInstance | null;
    root: ComponentInternalInstance;
    appContext: AppContext;
    /**
     * Vnode representing this component in its parent's vdom tree
     */
    vnode: VNode;
    /* Excluded from this release type: next */
    /**
     * Root vnode of this component's own vdom tree
     */
    subTree: VNode;
    /**
     * The reactive effect for rendering and patching the component. Callable.
     */
    update: ReactiveEffect;
    /* Excluded from this release type: render */
    /* Excluded from this release type: provides */
    /* Excluded from this release type: effects */
    /**
     * collection preMounted calllbacks, used in root component of unmounted subtree
     */
    preMountedCbs: Function[] | null;
    /* Excluded from this release type: accessCache */
    /* Excluded from this release type: renderCache */
    /* Excluded from this release type: components */
    /* Excluded from this release type: directives */
    proxy: ComponentPublicInstance | null;
    /* Excluded from this release type: withProxy */
    /* Excluded from this release type: ctx */
    data: Data;
    props: Data;
    attrs: Data;
    slots: InternalSlots;
    refs: Data;
    emit: EmitFn;
    /* Excluded from this release type: setupState */
    /* Excluded from this release type: setupContext */
    /* Excluded from this release type: suspense */
    /* Excluded from this release type: asyncDep */
    /* Excluded from this release type: asyncResolved */
    isMounted: boolean;
    isUnmounted: boolean;
    isDeactivated: boolean;
    /* Excluded from this release type: bc */
    /* Excluded from this release type: c */
    /* Excluded from this release type: bm */
    /* Excluded from this release type: bp */
    /* Excluded from this release type: p */
    /* Excluded from this release type: bpm */
    /* Excluded from this release type: pm */
    /* Excluded from this release type: m */
    /* Excluded from this release type: bu */
    /* Excluded from this release type: u */
    /* Excluded from this release type: bum */
    /* Excluded from this release type: um */
    /* Excluded from this release type: bre */
    /* Excluded from this release type: red */
    /* Excluded from this release type: rtc */
    /* Excluded from this release type: rtg */
    /* Excluded from this release type: a */
    /* Excluded from this release type: da */
    /* Excluded from this release type: ec */
    /* Excluded from this release type: renderUpdated */
}

export declare type ComponentObjectPropsOptions<P = Data> = {
    [K in keyof P]: Prop<P[K]> | null;
};

export declare type ComponentOptions = ComponentOptionsWithoutProps<any, any, any, any, any> | ComponentOptionsWithObjectProps<any, any, any, any, any> | ComponentOptionsWithArrayProps<any, any, any, any, any>;

export declare interface ComponentOptionsBase<Props, RawBindings, D, C extends ComputedOptions, M extends MethodOptions, E extends EmitsOptions, EE extends string = string> extends LegacyOptions<Props, D, C, M>, SFCInternalOptions, ComponentCustomOptions {
    setup?: (this: void, props: Props, ctx: SetupContext<E>) => RawBindings | RenderFunction | void;
    name?: string;
    template?: string | object;
    render?: Function;
    components?: Record<string, PublicAPIComponent>;
    directives?: Record<string, Directive>;
    inheritAttrs?: boolean;
    emits?: E | EE[];
    /* Excluded from this release type: ssrRender */
    /* Excluded from this release type: __asyncLoader */
    /* Excluded from this release type: __merged */
    call?: never;
    __isFragment?: never;
    __isTeleport?: never;
    __isSuspense?: never;
}

export declare type ComponentOptionsWithArrayProps<PropNames extends string = string, RawBindings = {}, D = {}, C extends ComputedOptions = {}, M extends MethodOptions = {}, E extends EmitsOptions = EmitsOptions, EE extends string = string, Props = Readonly<{
    [key in PropNames]?: any;
}>> = ComponentOptionsBase<Props, RawBindings, D, C, M, E, EE> & {
    props: PropNames[];
} & ThisType<ComponentPublicInstance<Props, RawBindings, D, C, M, E>>;

export declare type ComponentOptionsWithObjectProps<PropsOptions = ComponentObjectPropsOptions, RawBindings = {}, D = {}, C extends ComputedOptions = {}, M extends MethodOptions = {}, E extends EmitsOptions = EmitsOptions, EE extends string = string, Props = Readonly<ExtractPropTypes<PropsOptions>>> = ComponentOptionsBase<Props, RawBindings, D, C, M, E, EE> & {
    props: PropsOptions;
} & ThisType<ComponentPublicInstance<Props, RawBindings, D, C, M, E>>;

export declare type ComponentOptionsWithoutProps<Props = {}, RawBindings = {}, D = {}, C extends ComputedOptions = {}, M extends MethodOptions = {}, E extends EmitsOptions = EmitsOptions, EE extends string = string> = ComponentOptionsBase<Props, RawBindings, D, C, M, E, EE> & {
    props?: undefined;
} & ThisType<ComponentPublicInstance<{}, RawBindings, D, C, M, E, Readonly<Props>>>;

export declare type ComponentPropsOptions<P = Data> = ComponentObjectPropsOptions<P> | string[];

export declare type ComponentPublicInstance<P = {}, // props type extracted from props option
B = {}, // raw bindings returned from setup()
D = {}, // return from data()
C extends ComputedOptions = {}, M extends MethodOptions = {}, E extends EmitsOptions = {}, PublicProps = P> = {
    $: ComponentInternalInstance;
    $data: D;
    $props: PublicProps;
    $attrs: Data;
    $refs: Data;
    $slots: Slots;
    $root: ComponentPublicInstance | null;
    $parent: ComponentPublicInstance | null;
    $emit: EmitFn<E>;
    $el: any;
    $options: ComponentOptionsBase<P, B, D, C, M, E>;
    $forceUpdate: ReactiveEffect;
    $nextTick: typeof nextTick;
    $watch: typeof instanceWatch;
} & P & UnwrapRef<B> & D & ExtractComputedReturns<C> & M & ComponentCustomProperties;

declare type ComponentWatchOptionItem = WatchOptionItem | WatchOptionItem[];

declare type ComponentWatchOptions = Record<string, ComponentWatchOptionItem>;

export declare function computed<T>(getter: ComputedGetter<T>): ComputedRef<T>;

export declare function computed<T>(options: WritableComputedOptions<T>): WritableComputedRef<T>;

declare type ComputedOptions = Record<string, ComputedGetter<any> | WritableComputedOptions<any>>;
export { ComputedRef }

declare interface Constructor<P = any> {
    __isFragment?: never;
    __isTeleport?: never;
    __isSuspense?: never;
    new (): {
        $props: P;
    };
}

export declare type CreateAppFunction<HostElement> = (rootComponent: PublicAPIComponent, rootProps?: Data | null) => App<HostElement>;

/* Excluded from this release type: createBlock */

/* Excluded from this release type: createCommentVNode */

declare function createComponentInstance(vnode: VNode, parent: ComponentInternalInstance | null, suspense: SuspenseBoundary | null): ComponentInternalInstance;

export declare function createHydrationRenderer(options: RendererOptions<Node, Element>): HydrationRenderer;

declare function createRecord(id: string, comp: ComponentOptions): boolean;

/**
 * The createRenderer function accepts two generic arguments:
 * HostNode and HostElement, corresponding to Node and Element types in the
 * host environment. For example, for runtime-dom, HostNode would be the DOM
 * `Node` interface and HostElement would be the DOM `Element` interface.
 *
 * Custom renderers can pass in the platform specific types like this:
 *
 * ``` js
 * const { render, createApp } = createRenderer<Node, Element>({
 *   patchProp,
 *   ...nodeOps
 * })
 * ```
 */
export declare function createRenderer<HostNode = RendererNode, HostElement = RendererElement>(options: RendererOptions<HostNode, HostElement>): Renderer<HostElement>;

/* Excluded from this release type: createSlots */

/* Excluded from this release type: createStaticVNode */

/* Excluded from this release type: createTextVNode */

export declare const createVNode: typeof _createVNode;

declare function _createVNode(type: VNodeTypes | ClassComponent | typeof NULL_DYNAMIC_COMPONENT, props?: (Data & VNodeProps) | null, children?: unknown, patchFlag?: number, dynamicProps?: string[] | null, isBlockNode?: boolean): VNode;
export { customRef }

declare type Data = {
    [key: string]: unknown;
};
export { DebuggerEvent }

declare type DebuggerHook = (e: DebuggerEvent) => void;

declare type DefaultFactory<T> = () => T | null | undefined;

export declare function defineAsyncComponent<T extends PublicAPIComponent = {
    new (): ComponentPublicInstance;
}>(source: AsyncComponentLoader<T> | AsyncComponentOptions<T>): T;

export declare function defineComponent<Props, RawBindings = object>(setup: (props: Readonly<Props>, ctx: SetupContext) => RawBindings | RenderFunction): {
    new (): ComponentPublicInstance<Props, RawBindings, {}, {}, {}, VNodeProps & Props>;
} & FunctionalComponent<Props>;

export declare function defineComponent<Props = {}, RawBindings = {}, D = {}, C extends ComputedOptions = {}, M extends MethodOptions = {}, E extends EmitsOptions = Record<string, any>, EE extends string = string>(options: ComponentOptionsWithoutProps<Props, RawBindings, D, C, M, E, EE>): {
    new (): ComponentPublicInstance<Props, RawBindings, D, C, M, E, VNodeProps & Props>;
} & ComponentOptionsWithoutProps<Props, RawBindings, D, C, M, E, EE>;

export declare function defineComponent<PropNames extends string, RawBindings, D, C extends ComputedOptions = {}, M extends MethodOptions = {}, E extends EmitsOptions = Record<string, any>, EE extends string = string>(options: ComponentOptionsWithArrayProps<PropNames, RawBindings, D, C, M, E, EE>): {
    new (): ComponentPublicInstance<VNodeProps, RawBindings, D, C, M, E>;
} & ComponentOptionsWithArrayProps<PropNames, RawBindings, D, C, M, E, EE>;

export declare function defineComponent<PropsOptions extends Readonly<ComponentPropsOptions>, RawBindings, D, C extends ComputedOptions = {}, M extends MethodOptions = {}, E extends EmitsOptions = Record<string, any>, EE extends string = string>(options: ComponentOptionsWithObjectProps<PropsOptions, RawBindings, D, C, M, E, EE>): {
    new (): ComponentPublicInstance<ExtractPropTypes<PropsOptions>, RawBindings, D, C, M, E, VNodeProps & ExtractPropTypes<PropsOptions, false>>;
} & ComponentOptionsWithObjectProps<PropsOptions, RawBindings, D, C, M, E, EE>;

export declare type Directive<T = any> = ObjectDirective<T> | FunctionDirective<T>;

export declare type DirectiveArguments = Array<[Directive] | [Directive, any] | [Directive, any, string] | [Directive, any, string, DirectiveModifiers]>;

export declare interface DirectiveBinding {
    instance: ComponentPublicInstance | null;
    value: any;
    oldValue: any;
    arg?: string;
    modifiers: DirectiveModifiers;
    dir: ObjectDirective;
}

export declare type DirectiveHook<T = any, Prev = VNode<any, T> | null> = (el: T, binding: DirectiveBinding, vnode: VNode<any, T>, prevVNode: Prev) => void;

declare type DirectiveModifiers = Record<string, boolean>;

declare type EmitFn<Options = ObjectEmitsOptions, Event extends keyof Options = keyof Options> = Options extends any[] ? (event: Options[0], ...args: any[]) => void : UnionToIntersection<{
    [key in Event]: Options[key] extends ((...args: infer Args) => any) ? (event: key, ...args: Args) => void : (event: key, ...args: any[]) => void;
}[Event]>;

declare type EmitsOptions = ObjectEmitsOptions | string[];

declare type ErrorCapturedHook = (err: unknown, instance: ComponentPublicInstance | null, info: string) => boolean | void;

export declare const enum ErrorCodes {
    SETUP_FUNCTION = 0,
    RENDER_FUNCTION = 1,
    WATCH_GETTER = 2,
    WATCH_CALLBACK = 3,
    WATCH_CLEANUP = 4,
    NATIVE_EVENT_HANDLER = 5,
    COMPONENT_EVENT_HANDLER = 6,
    VNODE_HOOK = 7,
    DIRECTIVE_HOOK = 8,
    TRANSITION_HOOK = 9,
    APP_ERROR_HANDLER = 10,
    APP_WARN_HANDLER = 11,
    FUNCTION_REF = 12,
    ASYNC_COMPONENT_LOADER = 13,
    SCHEDULER = 14
}

declare type ErrorTypes = LifecycleHooks | ErrorCodes;

declare type ExtractComputedReturns<T extends any> = {
    [key in keyof T]: T[key] extends {
        get: Function;
    } ? ReturnType<T[key]['get']> : ReturnType<T[key]>;
};

export declare type ExtractPropTypes<O, MakeDefaultRequired extends boolean = true> = O extends object ? {
    [K in RequiredKeys<O, MakeDefaultRequired>]: InferPropType<O[K]>;
} & {
    [K in OptionalKeys<O, MakeDefaultRequired>]?: InferPropType<O[K]>;
} : {
    [K in string]: any;
};

export declare const Fragment: {
    new (): {
        $props: VNodeProps;
    };
    __isFragment: true;
};

export declare interface FunctionalComponent<P = {}, E extends EmitsOptions = Record<string, any>> extends SFCInternalOptions {
    (props: P, ctx: SetupContext<E>): any;
    props?: ComponentPropsOptions<P>;
    emits?: E | (keyof E)[];
    inheritAttrs?: boolean;
    displayName?: string;
}

export declare type FunctionDirective<T = any> = DirectiveHook<T>;

export declare const getCurrentInstance: () => ComponentInternalInstance | null;

export declare function h(type: string, children?: RawChildren): VNode;

export declare function h(type: string, props?: RawProps | null, children?: RawChildren): VNode;

export declare function h(type: typeof Fragment, children?: VNodeArrayChildren): VNode;

export declare function h(type: typeof Fragment, props?: RawProps | null, children?: VNodeArrayChildren): VNode;

export declare function h(type: typeof Teleport, props: RawProps & TeleportProps, children: RawChildren): VNode;

export declare function h(type: typeof Suspense, children?: RawChildren): VNode;

export declare function h(type: typeof Suspense, props?: (RawProps & SuspenseProps) | null, children?: RawChildren | RawSlots): VNode;

export declare function h<P>(type: FunctionalComponent<P>, props?: (RawProps & P) | ({} extends P ? null : never), children?: RawChildren | RawSlots): VNode;

export declare function h(type: Component, children?: RawChildren): VNode;

export declare function h<Options extends ComponentOptions | FunctionalComponent<{}>>(type: NotDefinedComponent<Options>, props?: RawProps | null, children?: RawChildren | RawSlots): VNode;

export declare function h(type: Constructor, children?: RawChildren): VNode;

export declare function h<P>(type: Constructor<P>, props?: (RawProps & P) | ({} extends P ? null : never), children?: RawChildren | RawSlots): VNode;

export declare function handleError(err: unknown, instance: ComponentInternalInstance | null, type: ErrorTypes): void;

export declare interface HMRRuntime {
    createRecord: typeof createRecord;
    rerender: typeof rerender;
    reload: typeof reload;
}

declare function hydrateSuspense(node: Node, vnode: VNode, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, isSVG: boolean, optimized: boolean, rendererInternals: RendererInternals, hydrateNode: (node: Node, vnode: VNode, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, optimized: boolean) => Node | null): Node | null;

declare function hydrateTeleport(node: Node, vnode: VNode, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, optimized: boolean, { o: { nextSibling, parentNode, querySelector } }: RendererInternals<Node, Element>, hydrateChildren: (node: Node | null, vnode: VNode, container: Element, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, optimized: boolean) => Node | null): Node | null;

export declare interface HydrationRenderer extends Renderer<Element> {
    hydrate: RootHydrateFunction;
}

declare type InferPropType<T> = T extends null ? any : T extends {
    type: null | true;
} ? any : T extends ObjectConstructor | {
    type: ObjectConstructor;
} ? {
    [key: string]: any;
} : T extends Prop<infer V> ? V : T;

export declare function inject<T>(key: InjectionKey<T> | string): T | undefined;

export declare function inject<T>(key: InjectionKey<T> | string, defaultValue: T): T;

export declare interface InjectionKey<T> extends Symbol {
}

declare function instanceWatch(this: ComponentInternalInstance, source: string | Function, cb: Function, options?: WatchOptions): WatchStopHandle;

/* Excluded from this release type: InternalRenderFunction */

declare type InternalSlots = {
    [name: string]: Slot | undefined;
};

declare type InvalidateCbRegistrator = (cb: () => void) => void;
export { isProxy }
export { isReactive }
export { isReadonly }
export { isRef }

declare function isVNode(value: any): value is VNode;

export declare const KeepAlive: new () => {
    $props: VNodeProps & KeepAliveProps;
};

export declare interface KeepAliveProps {
    include?: MatchPattern;
    exclude?: MatchPattern;
    max?: number | string;
}

declare interface LegacyOptions<Props, D, C extends ComputedOptions, M extends MethodOptions> {
    [key: string]: any;
    data?: (this: ComponentPublicInstance<Props>, vm: ComponentPublicInstance<Props>) => D;
    computed?: C;
    methods?: M;
    watch?: ComponentWatchOptions;
    provide?: Data | Function;
    inject?: ComponentInjectOptions;
    mixins?: ComponentOptions[];
    extends?: ComponentOptions;
    beforeCreate?(): void;
    created?(): void;
    beforeMount?(): void;
    beforeAppended?(): void;
    mounted?(): void;
    beforeUpdate?(): void;
    updated?(): void;
    activated?(): void;
    deactivated?(): void;
    beforeUnmount?(): void;
    unmounted?(): void;
    renderTracked?: DebuggerHook;
    renderTriggered?: DebuggerHook;
    errorCaptured?: ErrorCapturedHook;
}

declare type LifecycleHook = Function[] | null;

declare const enum LifecycleHooks {
    BEFORE_CREATE = "bc",
    CREATED = "c",
    BEFORE_MOUNT = "bm",
    BEFORE_PATCH = "bp",
    PATCHED = "p",
    PRE_MOUNTED = "pm",
    MOUNTED = "m",
    BEFORE_UPDATE = "bu",
    UPDATED = "u",
    BEFORE_UNMOUNT = "bum",
    UNMOUNTED = "um",
    DEACTIVATED = "da",
    ACTIVATED = "a",
    RENDER_TRIGGERED = "rtg",
    RENDER_TRACKED = "rtc",
    ERROR_CAPTURED = "ec",
    BEFORE_PRE_MOUNTED = "bpm",
    BEFORE_RENDER_EFFECT = "bre",
    RENDER_EFFECTED = "red"
}

declare type MapOldSources<T, Immediate> = {
    [K in keyof T]: T[K] extends WatchSource<infer V> ? Immediate extends true ? (V | undefined) : V : never;
};

declare type MapSources<T> = {
    [K in keyof T]: T[K] extends WatchSource<infer V> ? V : never;
};
export { markRaw }

declare type MatchPattern = string | RegExp | string[] | RegExp[];

/**
 * 在vue生成ssr时会用到（对atom-flutter没用），在cloneVNode时会用到
 * @param args
 */
export declare function mergeProps(...args: (Data & VNodeProps)[]): Data;

declare interface MethodOptions {
    [key: string]: Function;
}

declare type MountChildrenFn = (children: VNodeArrayChildren, container: RendererElement, anchor: RendererNode | null, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, isSVG: boolean, optimized: boolean, start?: number) => void;

declare type MountComponentFn = (initialVNode: VNode, container: RendererElement, anchor: RendererNode | null, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, isSVG: boolean, optimized: boolean) => void;

declare type MoveFn = (vnode: VNode, container: RendererElement, anchor: RendererNode | null, type: MoveType, parentSuspense?: SuspenseBoundary | null) => void;

declare function moveTeleport(vnode: VNode, container: RendererElement, parentAnchor: RendererNode | null, { o: { insert }, m: move }: RendererInternals, moveType?: TeleportMoveTypes): void;

declare const enum MoveType {
    ENTER = 0,
    LEAVE = 1,
    REORDER = 2
}

declare type NextFn = (vnode: VNode) => RendererNode | null;

export declare function nextTick(fn?: () => void): Promise<void>;

declare function normalizeSuspenseChildren(vnode: VNode): {
    content: VNode;
    fallback: VNode;
};

declare function normalizeVNode(child: VNodeChild): VNode;

declare type NotDefinedComponent<T extends Component> = T extends Constructor ? never : T;

declare const NULL_DYNAMIC_COMPONENT: unique symbol;

export declare interface ObjectDirective<T = any> {
    beforeMount?: DirectiveHook<T, null>;
    mounted?: DirectiveHook<T, null>;
    beforeUpdate?: DirectiveHook<T, VNode<any, T>>;
    updated?: DirectiveHook<T, VNode<any, T>>;
    beforeUnmount?: DirectiveHook<T, null>;
    unmounted?: DirectiveHook<T, null>;
    getSSRProps?: SSRDirectiveHook;
}

declare type ObjectEmitsOptions = Record<string, ((...args: any[]) => any) | null>;

export declare function onActivated(hook: Function, target?: ComponentInternalInstance | null): void;

export declare const onBeforeMount: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare const onBeforePreMounted: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare const onBeforeRenderEffect: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare const onBeforeUnmount: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare const onBeforeUpdate: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare function onDeactivated(hook: Function, target?: ComponentInternalInstance | null): void;

export declare const onErrorCaptured: (hook: ErrorCapturedHook, target?: ComponentInternalInstance | null) => void;

export declare const onMounted: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare const onPreMounted: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare const onRenderEffected: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare const onRenderTracked: (hook: DebuggerHook, target?: ComponentInternalInstance | null) => false | void;

export declare const onRenderTriggered: (hook: DebuggerHook, target?: ComponentInternalInstance | null) => false | void;

export declare const onUnmounted: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

export declare const onUpdated: (hook: () => any, target?: ComponentInternalInstance | null) => false | void;

/* Excluded from this release type: openBlock */

declare type OptionalKeys<T, MakeDefaultRequired> = Exclude<keyof T, RequiredKeys<T, MakeDefaultRequired>>;

export declare type OptionMergeFunction = (to: unknown, from: unknown, instance: any, key: string) => any;

declare type PatchBlockChildrenFn = (oldChildren: VNode[], newChildren: VNode[], fallbackContainer: RendererElement, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, isSVG: boolean) => void;

declare type PatchChildrenFn = (n1: VNode | null, n2: VNode, container: RendererElement, anchor: RendererNode | null, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, isSVG: boolean, optimized?: boolean) => void;

declare type PatchFn = (n1: VNode | null, // null means this is a mount
n2: VNode, container: RendererElement, anchor?: RendererNode | null, parentComponent?: ComponentInternalInstance | null, parentSuspense?: SuspenseBoundary | null, isSVG?: boolean, optimized?: boolean) => void;

declare type Plugin_2 = PluginInstallFunction & {
    install?: PluginInstallFunction;
} | {
    install: PluginInstallFunction;
};
export { Plugin_2 as Plugin }

declare type PluginInstallFunction = (app: App, ...options: any[]) => any;

/* Excluded from this release type: popScopeId */

export declare type Prop<T> = PropOptions<T> | PropType<T>;

declare type PropConstructor<T = any> = {
    new (...args: any[]): T & object;
} | {
    (): T;
} | PropMethod<T>;

declare type PropMethod<T> = T extends (...args: any) => any ? {
    new (): T;
    (): T;
    readonly proptotype: Function;
} : never;

declare interface PropOptions<T = any> {
    type?: PropType<T> | true | null;
    required?: boolean;
    default?: T | DefaultFactory<T> | null | undefined;
    validator?(value: unknown): boolean;
}

export declare type PropType<T> = PropConstructor<T> | PropConstructor<T>[];

export declare function provide<T>(key: InjectionKey<T> | string, value: T): void;

declare type PublicAPIComponent = Component | {
    new (...args: any[]): ComponentPublicInstance<any, any, any, any, any>;
};

/* Excluded from this release type: pushScopeId */

export declare function queuePostFlushCb(cb: Function | Function[]): void;

declare type RawChildren = string | number | boolean | VNode | VNodeArrayChildren | (() => any);

declare type RawProps = VNodeProps & {
    __v_isVNode?: never;
    [Symbol.iterator]?: never;
};

declare type RawSlots = {
    [name: string]: unknown;
    $stable?: boolean;
    _ctx?: ComponentInternalInstance | null;
    _?: 1;
};
export { reactive }
export { ReactiveEffect }
export { ReactiveEffectOptions }
export { readonly }
export { Ref }
export { ref }

/**
 * For runtime-dom to register the compiler.
 * Note the exported method uses any to avoid d.ts relying on the compiler types.
 */
export declare function registerRuntimeCompiler(_compile: any): void;

declare function reload(id: string, newComp: ComponentOptions): void;

declare type RemoveFn = (vnode: VNode) => void;

declare function renderComponentRoot(instance: ComponentInternalInstance): VNode;

export declare interface Renderer<HostElement = any> {
    render: RootRenderFunction<HostElement>;
    createApp: CreateAppFunction<HostElement>;
}

export declare interface RendererElement extends RendererNode {
}

declare interface RendererInternals<HostNode = RendererNode, HostElement = RendererElement> {
    p: PatchFn;
    um: UnmountFn;
    r: RemoveFn;
    m: MoveFn;
    mt: MountComponentFn;
    mc: MountChildrenFn;
    pc: PatchChildrenFn;
    pbc: PatchBlockChildrenFn;
    n: NextFn;
    o: RendererOptions<HostNode, HostElement>;
}

export declare interface RendererNode {
    [key: string]: any;
}

export declare interface RendererOptions<HostNode = RendererNode, HostElement = RendererElement> {
    patchProp(el: HostElement, key: string, prevValue: any, nextValue: any, isSVG?: boolean, prevChildren?: VNode<HostNode, HostElement>[], parentComponent?: ComponentInternalInstance | null, parentSuspense?: SuspenseBoundary | null, unmountChildren?: UnmountChildrenFn): void;
    insert(el: HostNode, parent: HostElement, anchor?: HostNode | null): void;
    remove(el: HostNode): void;
    createElement(type: string, isSVG?: boolean, isCustomizedBuiltIn?: string): HostElement;
    createText(text: string): HostNode;
    createComment(text: string): HostNode;
    setText(node: HostNode, text: string): void;
    setElementText(node: HostElement, text: string): void;
    parentNode(node: HostNode): HostElement | null;
    nextSibling(node: HostNode): HostNode | null;
    querySelector?(selector: string): HostElement | null;
    setScopeId?(el: HostElement, id: string): void;
    cloneNode?(node: HostNode): HostNode;
    insertStaticContent?(content: string, parent: HostElement, anchor: HostNode | null, isSVG: boolean): HostElement;
    setStaticContent?(node: HostElement, content: string): void;
    subTreePreMounted?(): void;
}

export declare type RenderFunction = () => VNodeChild;

/* Excluded from this release type: renderList */

/* Excluded from this release type: renderSlot */

declare type RequiredKeys<T, MakeDefaultRequired> = {
    [K in keyof T]: T[K] extends {
        required: true;
    } | (MakeDefaultRequired extends true ? {
        default: any;
    } : never) ? K : never;
}[keyof T];

declare function rerender(id: string, newRender?: Function): void;

export declare function resolveComponent(name: string): Component | string | undefined;

export declare function resolveDirective(name: string): Directive | undefined;

export declare function resolveDynamicComponent(component: unknown): Component | string | typeof NULL_DYNAMIC_COMPONENT;

export declare function resolveTransitionHooks(vnode: VNode, { appear, persisted, onBeforeEnter, onEnter, onAfterEnter, onEnterCancelled, onBeforeLeave, onLeave, onAfterLeave, onLeaveCancelled }: BaseTransitionProps<any>, state: TransitionState, instance: ComponentInternalInstance): TransitionHooks;

export declare type RootHydrateFunction = (vnode: VNode<Node, Element>, container: Element) => void;

export declare type RootRenderFunction<HostElement = RendererElement> = (vnode: VNode | null, container: HostElement) => void;

/* Excluded from this release type: setBlockTracking */

declare function setCurrentRenderingInstance(instance: ComponentInternalInstance | null): void;

export declare function setTransitionHooks(vnode: VNode, hooks: TransitionHooks): void;

declare function setupComponent(instance: ComponentInternalInstance, isSSR?: boolean): Promise<void> | undefined;

export declare interface SetupContext<E = ObjectEmitsOptions> {
    attrs: Data;
    slots: Slots;
    emit: EmitFn<E>;
}

declare type SetupRenderEffectFn = (instance: ComponentInternalInstance, initialVNode: VNode, container: RendererElement, anchor: RendererNode | null, parentSuspense: SuspenseBoundary | null, isSVG: boolean, optimized: boolean) => void;

declare interface SFCInternalOptions {
    /* Excluded from this release type: __scopeId */
    /* Excluded from this release type: __cssModules */
    /* Excluded from this release type: __hmrId */
    /* Excluded from this release type: __hmrUpdated */
    /**
     * This one should be exposed so that devtools can make use of it
     */
    __file?: string;
}
export { shallowReactive }
export { shallowReadonly }
export { shallowRef }

export declare type Slot = (...args: any[]) => VNode[];

export declare type Slots = Readonly<InternalSlots>;

export declare const ssrContextKey: unique symbol;

declare type SSRDirectiveHook = (binding: DirectiveBinding, vnode: VNode) => Data | undefined;

/* Excluded from this release type: ssrUtils */

export declare const Static: unique symbol;

export declare const Suspense: {
    new (): {
        $props: VNodeProps & SuspenseProps;
    };
    __isSuspense: true;
};

export declare interface SuspenseBoundary {
    vnode: VNode;
    parent: SuspenseBoundary | null;
    parentComponent: ComponentInternalInstance | null;
    isSVG: boolean;
    optimized: boolean;
    container: RendererElement;
    hiddenContainer: RendererElement;
    anchor: RendererNode | null;
    subTree: VNode;
    fallbackTree: VNode;
    deps: number;
    isHydrating: boolean;
    isResolved: boolean;
    isUnmounted: boolean;
    effects: Function[];
    preMountedCbs: Function[] | null;
    resolve(): void;
    recede(): void;
    move(container: RendererElement, anchor: RendererNode | null, type: MoveType): void;
    next(): RendererNode | null;
    registerDep(instance: ComponentInternalInstance, setupRenderEffect: SetupRenderEffectFn): void;
    unmount(parentSuspense: SuspenseBoundary | null, doRemove?: boolean): void;
}

declare const SuspenseImpl: {
    __isSuspense: boolean;
    process(n1: VNode<RendererNode, RendererElement> | null, n2: VNode<RendererNode, RendererElement>, container: RendererElement, anchor: RendererNode | null, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, isSVG: boolean, optimized: boolean, rendererInternals: RendererInternals<RendererNode, RendererElement>): void;
    hydrate: typeof hydrateSuspense;
};

export declare interface SuspenseProps {
    onResolve?: () => void;
    onRecede?: () => void;
}

export declare const Teleport: {
    new (): {
        $props: VNodeProps & TeleportProps;
    };
    __isTeleport: true;
};

declare const TeleportImpl: {
    __isTeleport: boolean;
    process(n1: VNode<RendererNode, RendererElement> | null, n2: VNode<RendererNode, RendererElement>, container: RendererElement, anchor: RendererNode | null, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, isSVG: boolean, optimized: boolean, internals: RendererInternals<RendererNode, RendererElement>): void;
    remove(vnode: VNode<RendererNode, RendererElement>, { r: remove, o: { remove: hostRemove } }: RendererInternals<RendererNode, RendererElement>): void;
    move: typeof moveTeleport;
    hydrate: typeof hydrateTeleport;
};

declare const enum TeleportMoveTypes {
    TARGET_CHANGE = 0,
    TOGGLE = 1,
    REORDER = 2
}

export declare interface TeleportProps {
    to: string | RendererElement;
    disabled?: boolean;
}

declare const Text_2: unique symbol;
export { Text_2 as Text }

/* Excluded from this release type: toDisplayString */

/* Excluded from this release type: toHandlers */
export { toRaw }
export { toRef }
export { ToRefs }
export { toRefs }
export { TrackOpTypes }

/* Excluded from this release type: transformVNodeArgs */

export declare interface TransitionHooks {
    persisted: boolean;
    beforeEnter(el: RendererElement): void;
    enter(el: RendererElement): void;
    leave(el: RendererElement, remove: () => void): void;
    afterLeave?(): void;
    delayLeave?(el: RendererElement, earlyRemove: () => void, delayedLeave: () => void): void;
    delayedLeave?(): void;
}

export declare interface TransitionState {
    isMounted: boolean;
    isLeaving: boolean;
    isUnmounting: boolean;
    leavingVNodes: Map<any, Record<string, VNode>>;
}
export { TriggerOpTypes }
export { triggerRef }

declare type UnionToIntersection<U> = (U extends any ? (k: U) => void : never) extends ((k: infer I) => void) ? I : never;

declare type UnmountChildrenFn = (children: VNode[], parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, doRemove?: boolean, start?: number) => void;

declare type UnmountFn = (vnode: VNode, parentComponent: ComponentInternalInstance | null, parentSuspense: SuspenseBoundary | null, doRemove?: boolean) => void;
export { unref }
export { UnwrapRef }

export declare const useCSSModule: (name?: string) => Record<string, string>;

export declare const useSSRContext: <T = Record<string, any>>() => T | undefined;

export declare function useTransitionState(): TransitionState;

export declare const version: string;

export declare interface VNode<HostNode = RendererNode, HostElement = RendererElement> {
    /* Excluded from this release type: __v_isVNode */
    /* Excluded from this release type: __v_skip */
    type: VNodeTypes;
    props: VNodeProps | null;
    key: string | number | null;
    ref: VNodeNormalizedRef | null;
    scopeId: string | null;
    children: VNodeNormalizedChildren;
    component: ComponentInternalInstance | null;
    suspense: SuspenseBoundary | null;
    dirs: DirectiveBinding[] | null;
    transition: TransitionHooks | null;
    el: HostNode | null;
    anchor: HostNode | null;
    target: HostElement | null;
    targetAnchor: HostNode | null;
    shapeFlag: number;
    patchFlag: number;
    dynamicProps: string[] | null;
    dynamicChildren: VNode[] | null;
    appContext: AppContext | null;
}

declare let vnodeArgsTransformer: ((args: Parameters<typeof _createVNode>, instance: ComponentInternalInstance | null) => Parameters<typeof _createVNode>) | undefined;

export declare interface VNodeArrayChildren extends Array<VNodeArrayChildren | VNodeChildAtom> {
}

export declare type VNodeChild = VNodeChildAtom | VNodeArrayChildren;

declare type VNodeChildAtom = VNode | string | number | boolean | null | undefined | void;

declare type VNodeMountHook = (vnode: VNode) => void;

export declare type VNodeNormalizedChildren = string | VNodeArrayChildren | RawSlots | null;

declare type VNodeNormalizedRef = [ComponentInternalInstance, VNodeRef];

export declare interface VNodeProps {
    [key: string]: any;
    key?: string | number;
    ref?: VNodeRef;
    onVnodeBeforeMount?: VNodeMountHook | VNodeMountHook[];
    onVnodeMounted?: VNodeMountHook | VNodeMountHook[];
    onVnodeBeforeUpdate?: VNodeUpdateHook | VNodeUpdateHook[];
    onVnodeUpdated?: VNodeUpdateHook | VNodeUpdateHook[];
    onVnodeBeforeUnmount?: VNodeMountHook | VNodeMountHook[];
    onVnodeUnmounted?: VNodeMountHook | VNodeMountHook[];
}

declare type VNodeRef = string | Ref | ((ref: object | null, refs: Record<string, any>) => void);

export declare type VNodeTypes = string | Component | typeof Text_2 | typeof Static | typeof Comment_2 | typeof Fragment | typeof TeleportImpl | typeof SuspenseImpl;

declare type VNodeUpdateHook = (vnode: VNode, oldVNode: VNode) => void;

export declare function warn(msg: string, ...args: any[]): void;

export declare function watch<T extends Readonly<WatchSource<unknown>[]>, Immediate extends Readonly<boolean> = false>(sources: T, cb: WatchCallback<MapSources<T>, MapOldSources<T, Immediate>>, options?: WatchOptions<Immediate>): WatchStopHandle;

export declare function watch<T, Immediate extends Readonly<boolean> = false>(source: WatchSource<T>, cb: WatchCallback<T, Immediate extends true ? (T | undefined) : T>, options?: WatchOptions<Immediate>): WatchStopHandle;

export declare function watch<T extends object, Immediate extends Readonly<boolean> = false>(source: T, cb: WatchCallback<T, Immediate extends true ? (T | undefined) : T>, options?: WatchOptions<Immediate>): WatchStopHandle;

export declare type WatchCallback<V = any, OV = any> = (value: V, oldValue: OV, onInvalidate: InvalidateCbRegistrator) => any;

export declare type WatchEffect = (onInvalidate: InvalidateCbRegistrator) => void;

export declare function watchEffect(effect: WatchEffect, options?: WatchOptionsBase): WatchStopHandle;

declare type WatchOptionItem = string | WatchCallback | {
    handler: WatchCallback;
} & WatchOptions;

export declare interface WatchOptions<Immediate = boolean> extends WatchOptionsBase {
    immediate?: Immediate;
    deep?: boolean;
}

export declare interface WatchOptionsBase {
    flush?: 'pre' | 'post' | 'sync';
    onTrack?: ReactiveEffectOptions['onTrack'];
    onTrigger?: ReactiveEffectOptions['onTrigger'];
}

export declare type WatchSource<T = any> = Ref<T> | ComputedRef<T> | (() => T);

export declare type WatchStopHandle = () => void;

/* Excluded from this release type: withCtx */

/**
 * Adds directives to a VNode.
 */
export declare function withDirectives<T extends VNode>(vnode: T, directives: DirectiveArguments): T;

/* Excluded from this release type: withScopeId */
export { WritableComputedOptions }

export { }

// Note: this file is auto concatenated to the end of the bundled d.ts during
// build.

declare module '@atom-vue/reactivity' {
  export interface RefUnwrapBailTypes {
    runtimeCoreBailTypes:
      | VNode
      | {
          // directly bailing on ComponentPublicInstance results in recursion
          // so we use this as a bail hint
          $: ComponentInternalInstance
        }
  }
}
